/////////////////////////////////////////////////////////////////////////////
// Name:        BurnOptions.cpp
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   XX
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "BurnOptions.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include "BurnOptions.h"

#include <wx/filename.h>
#include <wx/valgen.h>

// WDR: class implementations

//----------------------------------------------------------------------------
// CDataOptionsDlg
//----------------------------------------------------------------------------

// WDR: event table for CDataOptionsDlg

BEGIN_EVENT_TABLE(CDataOptionsDlg,wxDialog)
    EVT_CHECKBOX( ID_CHECKBOX_BURNLAYOUT_KEEP_IMAGES, CDataOptionsDlg::OnChangeKeepCDImage )
    EVT_BUTTON( ID_BUTTON_BURNLAYOUT_IMAGE_FILE_BROWSE, CDataOptionsDlg::OnBrowse )
END_EVENT_TABLE()

CDataOptionsDlg::CDataOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
    OptionsDataCD* pOptions,
    const wxPoint &position, const wxSize& size, long style ) :
    wxDialog( parent, id, title, position, size, style )
{
  DataOptionsDialog( this, TRUE );

  m_pOptions = pOptions;
  
  FindWindow(ID_COMBO_BURNLAYOUT_CD_DRIVE)->SetValidator(wxGenericValidator(m_pOptions->GetBurnDevicePtr()));
  FindWindow(ID_SPINCTRL_BURNLAYOUT_SPEED)->SetValidator(wxGenericValidator(m_pOptions->GetBurnSpeedPtr()));
  FindWindow(ID_TEXTCTRL_BURNLAYOUT_VOLUME_ID)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetVolumeIDPtr()));
  FindWindow(ID_SPINCTRL_BURNLAYOUT_FIFO)->SetValidator(wxGenericValidator(m_pOptions->GetFifoBufferPtr()));
  FindWindow(ID_CHECKBOX_BURNLAYOUT_FIXATE)->SetValidator(wxGenericValidator(m_pOptions->GetFixatePtr()));
  FindWindow(ID_CHECKBOX_BURNLAYOUT_EJECT)->SetValidator(wxGenericValidator(m_pOptions->GetEjectPtr()));
  FindWindow(ID_CHECKBOX_BURNLAYOUT_JOLIET)->SetValidator(wxGenericValidator(m_pOptions->GetJolietPtr()));
  FindWindow(ID_CHECKBOX_BURNLAYOUT_ROCKRIDGE)->SetValidator(wxGenericValidator(m_pOptions->GetRockridgePtr()));
  FindWindow(ID_CHECKBOX_BURNLAYOUT_SIMULATE)->SetValidator(wxGenericValidator(m_pOptions->GetSimulatePtr()));
  FindWindow(ID_CHECKBOX_BURNLAYOUT_KEEP_IMAGES)->SetValidator(wxGenericValidator(m_pOptions->GetKeepImagesPtr()));
  FindWindow(ID_TEXTCTRL_BURNLAYOUT_IMAGE_FILE)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetImageFilePtr()));
  FindWindow(ID_SPINCTRL_BURNLAYOUT_COPIES)->SetValidator(wxGenericValidator(m_pOptions->GetCopiesPtr()));
  
  TransferDataToWindow();
  
  // Turn off the image fields if the checkbox isn't marked
  if (!(m_pOptions->GetKeepImages()))
  {
    FindWindow(ID_TEXT_BURNLAYOUT_IMAGE_FILE)->Enable(FALSE);
    FindWindow(ID_TEXTCTRL_BURNLAYOUT_IMAGE_FILE)->Enable(FALSE);
    FindWindow(ID_BUTTON_BURNLAYOUT_IMAGE_FILE_BROWSE)->Enable(FALSE);
  }
}

void CDataOptionsDlg::PopulateCombo(BurnHelpers::BurnDeviceMap* pDeviceMap)
{
  wxComboBox* pCombo = (wxComboBox*)(FindWindow(ID_COMBO_BURNLAYOUT_CD_DRIVE));
  BurnHelpers::PopulateCDCombo(pDeviceMap, pCombo);
  TransferDataFromWindow();
}

// WDR: handler implementations for CDataOptionsDlg

void CDataOptionsDlg::OnBrowse( wxCommandEvent &event )
{
  TransferDataFromWindow();
  wxString strDefaultDir = "";
  wxString strDefaultFile = "";
  wxString strWildCard = "ISO files (*.iso)|*.iso|ALL files (*.*)|*.*";
  wxFileName file(m_pOptions->GetImageFile());
  
  // Check if the text in th text control is a directory
  if (file.DirExists())
  {
    strDefaultDir = m_pOptions->GetImageFile();
    strDefaultFile = "image";
  }
  else if (file.FileExists())
  {
    strDefaultDir = m_pOptions->GetImageFile().BeforeLast(wxFileName::GetPathSeparator());
    strDefaultFile = m_pOptions->GetImageFile().AfterLast(wxFileName::GetPathSeparator());
  }
  wxFileDialog dlg(this, "Choose a file", strDefaultDir, strDefaultFile, strWildCard, wxSAVE | wxOVERWRITE_PROMPT);
  if (dlg.ShowModal() == wxID_OK)
  {
    m_pOptions->SetImageFile(dlg.GetPath());
    ((wxTextCtrl*)FindWindow(ID_TEXTCTRL_BURNLAYOUT_IMAGE_FILE))->SetValue(dlg.GetPath());
  }
  TransferDataToWindow();
    
}

void CDataOptionsDlg::OnChangeKeepCDImage( wxCommandEvent &event )
{
  FindWindow(ID_TEXT_BURNLAYOUT_IMAGE_FILE)->Enable(event.IsChecked());
  FindWindow(ID_TEXTCTRL_BURNLAYOUT_IMAGE_FILE)->Enable(event.IsChecked());
  FindWindow(ID_BUTTON_BURNLAYOUT_IMAGE_FILE_BROWSE)->Enable(event.IsChecked());
}


//----------------------------------------------------------------------------
// CDataBurnOptionsDlg
//----------------------------------------------------------------------------

// WDR: event table for CDataBurnOptionsDlg

BEGIN_EVENT_TABLE(CDataBurnOptionsDlg,wxDialog)
    EVT_BUTTON( ID_BUTTON_BURNISO_IMAGE_FILE_BROWSE, CDataBurnOptionsDlg::OnBrowseIsoFile )
    EVT_BUTTON( wxID_OK, CDataBurnOptionsDlg::OnOK )
END_EVENT_TABLE()

CDataBurnOptionsDlg::CDataBurnOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
    OptionsDataCD* pOptions, const wxPoint &position, const wxSize& size, long style ) :
    wxDialog( parent, id, title, position, size, style )
{
  DataBurnOptionsDialog( this, TRUE );

  m_pOptions = pOptions;

  FindWindow(ID_SPINCTRL_BURN_SPEED)->SetValidator(wxGenericValidator(m_pOptions->GetBurnSpeedPtr()));
  FindWindow(ID_COMBO_CD_DRIVE)->SetValidator(wxGenericValidator(m_pOptions->GetBurnDevicePtr()));
  FindWindow(ID_SPINCTRL_FIFO)->SetValidator(wxGenericValidator(m_pOptions->GetFifoBufferPtr()));
  FindWindow(ID_CHECKBOX_EJECT)->SetValidator(wxGenericValidator(m_pOptions->GetEjectPtr()));
  FindWindow(ID_CHECKBOX_SIMULATE)->SetValidator(wxGenericValidator(m_pOptions->GetSimulatePtr()));
  FindWindow(ID_CHECKBOX_FIXATE)->SetValidator(wxGenericValidator(m_pOptions->GetFixatePtr()));
  FindWindow(ID_TEXTCTRL_BURNISO_IMAGE_FILE)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetImageFilePtr()));
  FindWindow(ID_SPINCTRL_BURNISO_COPIES)->SetValidator(wxGenericValidator(m_pOptions->GetCopiesPtr()));
  
  TransferDataToWindow();
}

void CDataBurnOptionsDlg::PopulateCombo(BurnHelpers::BurnDeviceMap* pDeviceMap)
{
  wxComboBox* pCombo = (wxComboBox*)(FindWindow(ID_COMBO_CD_DRIVE));
  BurnHelpers::PopulateCDCombo(pDeviceMap, pCombo);
  TransferDataFromWindow();
}

// WDR: handler implementations for CDataBurnOptionsDlg

void CDataBurnOptionsDlg::OnOK( wxCommandEvent &event )
{
  TransferDataFromWindow();

  // Make sure that an ISO file name was specified
  wxFileName file(m_pOptions->GetImageFile());
  if (!(file.FileExists()))
  {
    ::wxMessageBox(wxT("Please select an ISO image file"));
    FindWindow(ID_TEXTCTRL_BURNISO_IMAGE_FILE)->SetFocus();
    return;
  }
  // Make sure that a CD Burner was specified
  if (m_pOptions->GetBurnDevice().IsEmpty())
  {
    ::wxMessageBox(wxT("Please select a CD Writer"));
    FindWindow(ID_COMBO_CD_DRIVE)->SetFocus();
  }

  EndModal(wxID_OK);
}

void CDataBurnOptionsDlg::OnBrowseIsoFile( wxCommandEvent &event )
{
  TransferDataFromWindow();
  wxString strDefaultDir = "";
  wxString strDefaultFile = "";
  wxString strWildCard = "ISO files (*.iso)|*.iso|ALL files (*.*)|*.*";
  wxFileName file(m_pOptions->GetImageFile());
  
  // Check if the text in th text control is a directory
  if (file.DirExists())
  {
    strDefaultDir = m_pOptions->GetImageFile();
    strDefaultFile = "image";
  }
  else if (file.FileExists())
  {
    strDefaultDir = m_pOptions->GetImageFile().BeforeLast(wxFileName::GetPathSeparator());
    strDefaultFile = m_pOptions->GetImageFile().AfterLast(wxFileName::GetPathSeparator());
  }
  wxFileDialog dlg(this, "Choose a file", strDefaultDir, strDefaultFile, strWildCard, wxOPEN | wxFILE_MUST_EXIST);
  if (dlg.ShowModal() == wxID_OK)
  {
    m_pOptions->SetImageFile(dlg.GetPath());
    ((wxTextCtrl*)FindWindow(ID_TEXTCTRL_BURNISO_IMAGE_FILE))->SetValue(dlg.GetPath());
  }
  TransferDataToWindow();
}


//----------------------------------------------------------------------------
// CMakeIsoOptionsDlg
//----------------------------------------------------------------------------

// WDR: event table for CMakeIsoOptionsDlg

BEGIN_EVENT_TABLE(CMakeIsoOptionsDlg,wxDialog)
    EVT_BUTTON( ID_BUTTON_MAKEISO_IMAGE_FILE_BROWSE, CMakeIsoOptionsDlg::OnBrowseIsoFile )
    EVT_BUTTON( wxID_OK, CMakeIsoOptionsDlg::OnOK )
END_EVENT_TABLE()

CMakeIsoOptionsDlg::CMakeIsoOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
    OptionsDataCD* pOptions, const wxPoint &position, const wxSize& size, long style ) :
    wxDialog( parent, id, title, position, size, style )
{
  MakeIsoOptionsDialog( this, TRUE );
  
  m_pOptions = pOptions;
  FindWindow(ID_TEXTCTRL_VOLUME_ID)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetVolumeIDPtr()));
  FindWindow(ID_TEXTCTRL_MAKEISO_IMAGE_FILE)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetImageFilePtr()));
  FindWindow(ID_CHECKBOX_JOLIET)->SetValidator(wxGenericValidator(m_pOptions->GetJolietPtr()));
  FindWindow(ID_CHECKBOX_ROCKRIDGE)->SetValidator(wxGenericValidator(m_pOptions->GetRockridgePtr()));
  
  TransferDataToWindow();
}

// WDR: handler implementations for CMakeIsoOptionsDlg

void CMakeIsoOptionsDlg::OnOK( wxCommandEvent &event )
{
  TransferDataFromWindow();

  // Make sure that an ISO file name was specified
  wxString strImage = m_pOptions->GetImageFile();
  strImage.Trim();
  if (strImage.IsEmpty())
  {
    ::wxMessageBox(wxT("Please select an ISO image file"));
    FindWindow(ID_TEXTCTRL_MAKEISO_IMAGE_FILE)->SetFocus();
    return;
  }

  EndModal(wxID_OK);
}

void CMakeIsoOptionsDlg::OnBrowseIsoFile( wxCommandEvent &event )
{
  TransferDataFromWindow();
  wxString strDefaultDir = "";
  wxString strDefaultFile = "";
  wxString strWildCard = "ISO files (*.iso)|*.iso|ALL files (*.*)|*.*";
  wxFileName file(m_pOptions->GetImageFile());
  
  // Check if the text in th text control is a directory
  if (file.DirExists())
  {
    strDefaultDir = m_pOptions->GetImageFile();
    strDefaultFile = "image";
  }
  else if (file.FileExists())
  {
    strDefaultDir = m_pOptions->GetImageFile().BeforeLast(wxFileName::GetPathSeparator());
    strDefaultFile = m_pOptions->GetImageFile().AfterLast(wxFileName::GetPathSeparator());
  }
  wxFileDialog dlg(this, "Choose a file", strDefaultDir, strDefaultFile, strWildCard, wxSAVE | wxOVERWRITE_PROMPT);
  if (dlg.ShowModal() == wxID_OK)
  {
    m_pOptions->SetImageFile(dlg.GetPath());
    ((wxTextCtrl*)FindWindow(ID_TEXTCTRL_MAKEISO_IMAGE_FILE))->SetValue(dlg.GetPath());
  }
  TransferDataToWindow();
}


//----------------------------------------------------------------------------
// CDataDefaultOptionsDlg
//----------------------------------------------------------------------------

// WDR: event table for CDataDefaultOptionsDlg

BEGIN_EVENT_TABLE(CDataDefaultOptionsDlg,wxDialog)
    EVT_BUTTON( ID_BUTTON_DEFAULT_IMAGE_FILE_BROWSE, CDataDefaultOptionsDlg::OnBrowseIsoFile )
END_EVENT_TABLE()

CDataDefaultOptionsDlg::CDataDefaultOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
    OptionsDataCD* pOptions, const wxPoint &position, const wxSize& size, long style ) :
    wxDialog( parent, id, title, position, size, style )
{
  DataDefaultOptionsDialog( this, TRUE );

  m_pOptions = pOptions;
  
  FindWindow(ID_SPINCTRL_DEFAULT_BURN_SPEED)->SetValidator(wxGenericValidator(m_pOptions->GetBurnSpeedPtr()));
  FindWindow(ID_COMBO_DEFAULT_CD_DRIVE)->SetValidator(wxGenericValidator(m_pOptions->GetBurnDevicePtr()));
  FindWindow(ID_TEXTCTRL_DEFAULT_VOLUME_ID)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetVolumeIDPtr()));
  FindWindow(ID_TEXTCTRL_DEFAULT_IMAGE_FILE)->SetValidator(wxTextValidator(wxFILTER_ASCII, m_pOptions->GetImageFilePtr()));
  FindWindow(ID_SPINCTRL_DEFAULT_FIFO)->SetValidator(wxGenericValidator(m_pOptions->GetFifoBufferPtr()));
  FindWindow(ID_CHECKBOX_DEFAULT_EJECT)->SetValidator(wxGenericValidator(m_pOptions->GetEjectPtr()));
  FindWindow(ID_CHECKBOX_DEFAULT_JOLIET)->SetValidator(wxGenericValidator(m_pOptions->GetJolietPtr()));
  FindWindow(ID_CHECKBOX_DEFAULT_ROCKRIDGE)->SetValidator(wxGenericValidator(m_pOptions->GetRockridgePtr()));
  FindWindow(ID_CHECKBOX_DEFAULT_SIMULATE)->SetValidator(wxGenericValidator(m_pOptions->GetSimulatePtr()));
  FindWindow(ID_CHECKBOX_DEFAULT_FIXATE)->SetValidator(wxGenericValidator(m_pOptions->GetFixatePtr()));
  
  TransferDataToWindow();
}

void CDataDefaultOptionsDlg::PopulateCombo(BurnHelpers::BurnDeviceMap* pDeviceMap)
{
  wxComboBox* pCombo = (wxComboBox*)(FindWindow(ID_COMBO_DEFAULT_CD_DRIVE));
  BurnHelpers::PopulateCDCombo(pDeviceMap, pCombo);
  TransferDataFromWindow();
}

// WDR: handler implementations for CDataDefaultOptionsDlg

void CDataDefaultOptionsDlg::OnBrowseIsoFile( wxCommandEvent &event )
{
  TransferDataFromWindow();
  wxString strDefaultDir = "";
  wxString strDefaultFile = "";
  wxString strWildCard = "ISO files (*.iso)|*.iso|ALL files (*.*)|*.*";
  wxFileName file(m_pOptions->GetImageFile());
  
  // Check if the text in th text control is a directory
  if (file.DirExists())
  {
    strDefaultDir = m_pOptions->GetImageFile();
    strDefaultFile = "image";
  }
  else if (file.FileExists())
  {
    strDefaultDir = m_pOptions->GetImageFile().BeforeLast(wxFileName::GetPathSeparator());
    strDefaultFile = m_pOptions->GetImageFile().AfterLast(wxFileName::GetPathSeparator());
  }
  wxFileDialog dlg(this, "Choose a file", strDefaultDir, strDefaultFile, strWildCard, wxSAVE | wxOVERWRITE_PROMPT);
  if (dlg.ShowModal() == wxID_OK)
  {
    m_pOptions->SetImageFile(dlg.GetPath());
    ((wxTextCtrl*)FindWindow(ID_TEXTCTRL_DEFAULT_IMAGE_FILE))->SetValue(dlg.GetPath());
  }
  TransferDataToWindow();
}

BurnOptions::BurnOptions()
{
  m_nBurnSpeed = 40;
  m_strBurnDevice = "";
  m_bSimulate = FALSE;
  m_bEject = TRUE;
  m_strImageFile = "";
  m_nCopies = 1;
}

OptionsDataCD::OptionsDataCD() :
  BurnOptions()
{
  m_strVolumeID = "DATACD";
  m_bJoliet = TRUE;
  m_bRockridge = TRUE;
  
  m_nFifoBuffer = 254;
  m_bFixate = TRUE;
}

OptionsVideoCD::OptionsVideoCD() :
  BurnOptions()
{
}

OptionsAudioCD::OptionsAudioCD() :
  BurnOptions()
{
}
