/////////////////////////////////////////////////////////////////////////////
// Name:        BurnProgressDlg.h
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   XX
/////////////////////////////////////////////////////////////////////////////

#ifndef __BurnProgressDlg_H__
#define __BurnProgressDlg_H__

#ifdef __GNUG__
    #pragma interface "BurnProgressDlg.cpp"
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/process.h>

#include "wxBurn_wdr.h"
#include "BurnOptions.h"
#include "CDLayoutTreeCtrl.h"

// WDR: class declarations

//----------------------------------------------------------------------------
// BurnProgressDlg
//----------------------------------------------------------------------------

class BurnProgressDlg : public wxDialog
{
public:
  // constructors and destructors
  BurnProgressDlg(wxWindow* parent, wxWindowID id, const wxString& title,
        const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
        long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = "dialogBox")
     : wxDialog(parent, id, title, pos, size, style, name)
      { 
        SetMode(CREATE_IMAGE);
        m_bCreateImage = FALSE;
        m_bBurnImage = FALSE;
      }

  enum FeedBackMode
  {
   CREATE_IMAGE = 0,
    BURN_IMAGE
  };
    
  // WDR: method declarations for BurnProgressDlg
  virtual void SetBurnOptions(BurnOptions* pOptions) { m_pOptions = pOptions; }
  virtual wxString GenerateCmdLine() = 0;
  virtual void LoadDialog() { FeedbackDialog(this); }
  virtual int ProcessCmd();
  virtual void ProcessLine(const wxString& strLine);
  virtual void SetMode(int nMode) { m_nMode = nMode; }
  virtual int GetMode() const { return m_nMode; }
  static wxString ConvertToMountPoint(const wxString& strOriginalPath);
  virtual void DisplayErrorDialog(int nErrorCode);
  virtual void DisplayProcessLog();
  
protected:
  // WDR: member variable declarations for BurnProgressDlg
  BurnOptions*    m_pOptions;
  int             m_nMode;    // This should correspond to a type in FeedBackMode above.  This will be needed for GenerateCmdLine.
  static int      m_nLastReturnCode; // This is a hack to get the return code of the last operation
  static bool     m_bInProcess;     // This is a hack to know if we're waiting for a process to end
  bool            m_bCreateImage;
  bool            m_bBurnImage;
  wxArrayString   m_strOutputArray;

private:
  // WDR: handler declarations for BurnProgressDlg
  void OnStart(wxCommandEvent &event);
  void OnEndProcess(wxProcessEvent& event);

private:
  DECLARE_EVENT_TABLE()
};


//----------------------------------------------------------------------------
// DataBurnProgressDlg
//----------------------------------------------------------------------------

// DataBurnProgressDlg will use mkisofs for CD image creation
//  and cdrecord for CD image burning
class DataBurnProgressDlg : public BurnProgressDlg
{
public:
  // constructors and destructors
  DataBurnProgressDlg(wxWindow* parent, wxWindowID id, const wxString& title,
      const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
      long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = "dialogBox")
   : BurnProgressDlg(parent, id, title, pos, size, style, name) { m_pLayoutMap = NULL; m_pLayoutExcludeMap = NULL; }

  // WDR: method declarations for DataBurnProgressDlg
  virtual void SetBurnOptions(BurnOptions* pOptions)
  {
    //wxASSERT(pOptions->IsKindOf(CLASSINFO(OptionsDataCD)));
    BurnProgressDlg::SetBurnOptions(pOptions);
  }

  // If the strIsoImage is blank, the assume that we want to ask the user for the ISO image
  // CreateISOImage will use an "Save As" theme and BurnISOImage will use an "Open File" theme
  void CreateISOImage(const wxString& strIsoImage = "");
  void BurnISOImage(const wxString& strIsoImage = "");
  void BurnCDLayout();
    
  virtual wxString GenerateCmdLine();
  virtual void LoadDialog();
  virtual void ProcessLine(const wxString& strLine);
  void SetLayoutMaps(CDLayoutMap* pMap, CDLayoutMap* pExcludeMap) { m_pLayoutMap = pMap; m_pLayoutExcludeMap = pExcludeMap; }

protected:
    // WDR: member variable declarations for DataBurnProgressDlg
  CDLayoutMap* m_pLayoutMap;
  CDLayoutMap* m_pLayoutExcludeMap;

  // Feedback dialog variables
  wxGauge* m_pGauge;
  wxStaticText* m_pCurrentOperationText;
  wxStaticText* m_pStatusText;
  long m_nTotalTime;
  long m_nTotalAmt;

private:
  // WDR: handler declarations for DataBurnProgressDlg
  void OnStart(wxCommandEvent& event);

private:
  DECLARE_EVENT_TABLE()
};


//----------------------------------------------------------------------------
// DaoBurnProgressDlg
//----------------------------------------------------------------------------

// DaoBurnProgressDlg will use cdrdao for CD creation
class DaoBurnProgressDlg : public BurnProgressDlg
{
public:
    // constructors and destructors
    DaoBurnProgressDlg(wxWindow* parent, wxWindowID id, const wxString& title,
            const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
            long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = "dialogBox")
        : BurnProgressDlg(parent, id, title, pos, size, style, name) { }

    // WDR: method declarations for DaoBurnProgressDlg
    virtual void SetBurnOptions(BurnOptions* pOptions)
    {
//      wxASSERT(pOptions->IsKindOf(CLASSINFO(OptionsDataCD)));
        BurnProgressDlg::SetBurnOptions(pOptions);
    }

    // If the strIsoImage is blank, the assume that we want to ask the user for the ISO image
    // CreateISOImage will use an "Save As" theme and BurnISOImage will use an "Open File" theme
    void CreateBinImage(const wxString& strIsoImage = "");
    void BurnBinImage(const wxString& strIsoImage = "");
    
    virtual wxString GenerateCmdLine();

protected:
    // WDR: member variable declarations for DaoBurnProgressDlg

private:
    // WDR: handler declarations for DaoBurnProgressDlg

private:
  DECLARE_EVENT_TABLE()
};


//----------------------------------------------------------------------------
// VideoBurnProgressDlg
//----------------------------------------------------------------------------

// VideoBurnProgressDlg will use VCDImager to create the CD image and
//  let the DaoBurnProgressDlg base class use cdrdao to burn the image
class VideoBurnProgressDlg : public DaoBurnProgressDlg
{
public:
    // constructors and destructors
    VideoBurnProgressDlg(wxWindow* parent, wxWindowID id, const wxString& title,
            const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
            long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = "dialogBox")
        : DaoBurnProgressDlg(parent, id, title, pos, size, style, name) { }

    // WDR: method declarations for VideoBurnProgressDlg
    virtual void SetBurnOptions(BurnOptions* pOptions)
    {
//      wxASSERT(pOptions->IsKindOf(CLASSINFO(OptionsVideoCD)));
        BurnProgressDlg::SetBurnOptions(pOptions);
    }

private:
    // WDR: member variable declarations for VideoBurnProgressDlg

private:
    // WDR: handler declarations for VideoBurnProgressDlg

private:
  DECLARE_EVENT_TABLE()
};



//----------------------------------------------------------------------------
// AudioBurnProgressDlg
//----------------------------------------------------------------------------

// AudioBurnProgressDlg will use cdrecord for CD creation.  I'm not sure what to use for CUE/BIN creation though
class AudioBurnProgressDlg : public BurnProgressDlg
{
public:
    // constructors and destructors
    AudioBurnProgressDlg(wxWindow* parent, wxWindowID id, const wxString& title,
            const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
            long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = "dialogBox")
        : BurnProgressDlg(parent, id, title, pos, size, style, name) { }

    // WDR: method declarations for AudioBurnProgressDlg
    virtual void SetBurnOptions(BurnOptions* pOptions)
    {
//      wxASSERT(pOptions->IsKindOf(CLASSINFO(OptionsAudioCD)));
        BurnProgressDlg::SetBurnOptions(pOptions);
    }

private:
    // WDR: member variable declarations for AudioBurnProgressDlg

private:
    // WDR: handler declarations for AudioBurnProgressDlg

private:
  DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
// CDCopyProgressDlg
//----------------------------------------------------------------------------

// CDCopyProgressDlg will use base class DaoBurnProgressDlg's functions for reading and writing BIN/CUE files
//  when doing multiple copies, and use cdrdao's copy command when making only one copy
class CDCopyProgressDlg : public DaoBurnProgressDlg
{
public:
    // constructors and destructors
    CDCopyProgressDlg(wxWindow* parent, wxWindowID id, const wxString& title,
            const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
            long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = "dialogBox")
        : DaoBurnProgressDlg(parent, id, title, pos, size, style, name)
    {
      m_pGauge = NULL; m_pCurrentOperationText = NULL; m_pStatusText = NULL;
      m_nTotalTime = 0; m_nTotalAmt = 0;
    }

    // WDR: method declarations for CDCopyProgressDlg
    virtual void SetBurnOptions(BurnOptions* pOptions)
    {
//      wxASSERT(pOptions->IsKindOf(CLASSINFO(CCopyCDOptions)));
        BurnProgressDlg::SetBurnOptions(pOptions);
    }

    void CopyCD();
    
    virtual wxString GenerateCmdLine();
    virtual void LoadDialog() { CopyCDFeedbackDialog(this); }
    virtual void ProcessLine(const wxString& strLine);

protected:
    // WDR: member variable declarations for CDCopyProgressDlg
    wxGauge* m_pGauge;
    wxStaticText* m_pCurrentOperationText;
    wxStaticText* m_pStatusText;
    long m_nTotalTime;
    long m_nTotalAmt;

private:
    // WDR: handler declarations for CDCopyProgressDlg
    void OnStart(wxCommandEvent &event);

private:
  DECLARE_EVENT_TABLE()
};


/// Output Line Formats ///
/*
  mkisofs:
      Scanning /X/X
      Using X;# for  X/X (X)
      XXX.XX% done, estimate finish Fri Apr 11 10:45:04 2003
      Total translation table size: #
      Total rockridge attributes bytes: #####
      Total directory bytes: #####
      Path table size(bytes): ###
      Max brk space used ####
      #### extents written (### Mb)

  cdrecord:
      Track ##:   # of # MB written (fifo  #%).
      Track ##: Total bytes read/written: #/# (# sectors).
      Writing  time:  X.XXXs
      Fixating...
      Fixating time:   X.XXXs

  cdrdao read-cd:
      Reading toc and track data...
      Track   Mode    Flags  Start                Length
      ------------------------------------------------------------
       1      DATA    4      00:00:00(     0)     04:56:52( 22252)
      Leadout DATA    4      04:56:52( 22252)
      Copying data track 1 (MODE1_RAW): start 00:00:00, length 04:56:52 to "test.bin"...
      00:01:00
      00:02:00
      ...
      04:56:00

  cdrdao write:
      Starting write simulation at speed 32...
      Pausing 10 seconds - hit CTRL-C to abort.
      Process can be aborted with QUIT signal (usually CTRL-\).
      Turning BURN-Proof on
      Writing track 01 (mode MODE1_RAW/MODE1_RAW )...
      Wrote 1 of 49 MB (Buffer 100%).
      Wrote 2 of 49 MB (Buffer 100%).
      ...
      Wrote 49 of 49 MB (Buffer 100%).
      Wrote 22252 blocks. Buffer fill min 100%/max 100%.
      Flushing cache...
      Simulation finished successfully.

  cdrdao copy:
      Starting CD copy simulation at speed 32...
      Track   Mode    Flags  Start                Length
      ------------------------------------------------------------
       1      DATA    4      00:00:00(     0)     04:56:52( 22252)
      Leadout DATA    4      04:56:52( 22252)
      Copying data track 1 (MODE1_RAW): start 00:00:00, length 04:56:52 to "test.bin"...
      00:01:00
      00:02:00
      ...
      04:56:00
      Keeping created image file "test.bin".
      Corresponding toc-file is written to "cd1156.toc".
      Turning BURN-Proof on
      Writing track 01 (mode MODE1_RAW/MODE1_RAW )...
      Wrote 1 of 49 MB (Buffer 100%).
      ...
      Wrote 49 of 49 MB (Buffer 100%).
      Wrote 22252 blocks. Buffer fill min 100%/max 100%.
      Flushing cache...
      Simulation finished successfully.
      CD copying finished successfully.
*/
#endif
