#include "CDLayoutTreeCtrl.h"
#include "wxBurn_wdr.h"
#include "wxBurn.h"

#include <sys/types.h>
#include <sys/stat.h>

#include <wx/filename.h>
#include <wx/filefn.h>
#include <wx/busyinfo.h>

//------------------------------------------------------------------------------
// CDLayoutTreeCtrl
//------------------------------------------------------------------------------
CDLayoutTreeCtrl::CDLayoutTreeCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pos /*= wxDefaultPosition*/,
      const wxSize& size /*= wxDefaultSize*/, long style /*= wxTR_HAS_BUTTONS*/,
      const wxValidator& validator /*= wxDefaultValidator*/, const wxString& name /*= "listCtrl"*/) : 
  wxTreeCtrl(parent, id, pos, size, style, validator, name)
{
  m_CDLayoutMap.clear();
  m_CDLayoutExcludeMap.clear();
	CDLayoutItem item("one", "two", CDLayoutItem::LAYOUT_FILE);
	m_CDLayoutExcludeMap["s"] = item;
	m_nTotalSize = 0;
}

CDLayoutItem& CDLayoutItem::operator=(const CDLayoutItem& item)
{
  m_strLayoutPosition = item.m_strLayoutPosition;
  m_strOrigin = item.m_strOrigin;
  m_nFileType = item.m_nFileType;

  return *this;
}

wxString CDLayoutTreeCtrl::GetCurrentCDLayoutTreePath()
{
  wxTreeItemId TreeItem = GetSelection();
  if (!(TreeItem.IsOk())) // Make sure that an item is selected
  {
    // Use the root node if nothing was selected
    return GetItemText(GetRootItem());
  }
  wxString strPath;// = GetItemText(TreeItem);
  wxChar strSeparator = wxFileName::GetPathSeparator();

  while (GetRootItem() != TreeItem)
  {
    // Prepend the path separator to the beginning of the path
    strPath.Prepend(strSeparator);

    // Get the directory name
    strPath.Prepend(GetItemText(TreeItem));

    // Move up in the tree
    TreeItem = GetItemParent(TreeItem);
  }

  // Put one last "/" or "\" at the beginning of the path
  strPath.Prepend(strSeparator);
  return strPath;
}

wxString CDLayoutTreeCtrl::GetCDLayoutTreePathFromCoord(wxCoord x, wxCoord y)
{
  int flags = wxTREE_HITTEST_ONITEMICON;
  wxTreeItemId TreeItem = HitTest(wxPoint(x, y), flags);
  if (!(TreeItem.IsOk()))  // Did we land on an item??
  {
    // in this case we didn't land on an item so just return GetCurrentCDLayoutTreePath()
    return GetCurrentCDLayoutTreePath();
  }
  else
  {
		SelectItem(TreeItem);

    wxString strPath;// = GetItemText(TreeItem);
    wxChar strSeparator = wxFileName::GetPathSeparator();

    while (GetRootItem() != TreeItem)
    {
      // Prepend the path separator to the beginning of the path
      strPath.Prepend(strSeparator);

      // Get the directory name
      strPath.Prepend(GetItemText(TreeItem));

      // Move up in the tree
      TreeItem = GetItemParent(TreeItem);
    }

    // Put one last "/" or "\" at the beginning of the path
    strPath.Prepend(strSeparator);
    return strPath;
  }
}

void CDLayoutTreeCtrl::SelectItemFromHitTest(const wxPoint& pt)
{
  int flags = wxTREE_HITTEST_ONITEMICON;
  wxTreeItemId TreeItem = HitTest(pt, flags);
  if (TreeItem.IsOk())  // Did we land on an item??
	{
		SelectItem(TreeItem);
	}
}

/*
void CDLayoutTreeCtrl::AddToCDLayout(CDLayoutItem& item, wxListCtrl* pListCtrl)
{
//  ::wxMessageBox(wxString::Format(wxT("Layout Position: %s\nOrigin%s\nFile Or Directory: %d\nInclude: %d"),
//          item.GetLayoutPosition().c_str(), item.GetOrigin().c_str(), item.GetFileType(), item.GetInclude()));
}
*/
void CDLayoutTreeCtrl::AddToCDLayout(wxString& strPath, wxListCtrl* pListCtrl, const wxString& strTreePath /* = "" */)
{
  // Check for entries already in the class array (described below) with the same "CD file layout position/name"
  // Add the entry as a new instance of a class that saves 
  //    "CD file layout position/name",
  //    "source path", "whether file or directory",
  //    "whether to include or exclude"
  // An array of these classes will be used to populate the tree control which will populate the list control
  int nFileType = CDLayoutItem::LAYOUT_FILE;
  if (wxFileName::DirExists(strPath))
  {
   nFileType = CDLayoutItem::LAYOUT_DIRECTORY; // This is a directory
  }
  else if (wxFileName::FileExists(strPath))
  {
   nFileType = CDLayoutItem::LAYOUT_FILE; // This is a file
  }
  
  wxString strFilePath = (strTreePath != "") ? strTreePath : GetCurrentCDLayoutTreePath();
  // If this CD Layout tree node is the root node and this is a directory, then check if this node should be put at the root directory
  if ((nFileType == CDLayoutItem::LAYOUT_DIRECTORY) &&
      (strFilePath == wxFileName::GetPathSeparator()) &&
      (m_CDLayoutMap.find(wxFileName::GetPathSeparator()) == m_CDLayoutMap.end())) // Don't ask if there is already a folder set to root
  {
    if (wxYES == ::wxMessageBox("Would you like to set " + strPath + " as the root directory?", "CD Layout", wxYES_NO))
    {
      strFilePath = wxFileName::GetPathSeparator();
    }
    else
    {
      strFilePath +=  strPath.AfterLast(wxFileName::GetPathSeparator());
    }
  }
  else
  {
    strFilePath +=  strPath.AfterLast(wxFileName::GetPathSeparator());
  }

  CDLayoutItem item(strFilePath, strPath, nFileType);

  if ((m_CDLayoutMap.size() == 0) || (m_CDLayoutMap.find(item.GetLayoutPosition()) == m_CDLayoutMap.end()))
  {
     m_CDLayoutMap[item.GetLayoutPosition()] = item;

    // Add the file size to the size used
    if (nFileType == CDLayoutItem::LAYOUT_DIRECTORY)
    {
      // Add this directory tree to the layout tree and list control
      if (strFilePath != wxFileName::GetPathSeparator())  // Only create a new tree item if this wasn't selected as the root node
      {
        wxTreeItemId TreeItem = GetSelection();
        if (!(TreeItem.IsOk())) // Make sure that an item is selected
        {
          // Use the root node if nothing was selected
          TreeItem = GetRootItem();
        }
        SelectItem(AppendItem(TreeItem, strPath.AfterLast(wxFileName::GetPathSeparator()), 0, 1));
      }
      PopulateTreeAndListFromPath(item, pListCtrl);

      wxBusyInfo* pWait = new wxBusyInfo("Calculating directory disk usage...");
      long nDirSize = GetDiskSpaceUsed(strPath);
      delete pWait;
      /*
      wxDir dir;
      dir.Open(strPath);
	    long nDirSize = 0;
      CDLayoutSpaceTraverser DirTraverse(&nDirSize);
      dir.Traverse(DirTraverse);
      */
	    AddFileSize(nDirSize);
    }
    else
    {
      // Add this item to the list control
      wxFileName file(item.GetOrigin());
      int nIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), file.GetFullName(), (nFileType == CDLayoutItem::LAYOUT_DIRECTORY) ? 0 : 1);
      pListCtrl->SetItem(nIndex, 2, item.GetOrigin());

	    wxStructStat FileStat;
	    if (::wxStat(strPath, &FileStat) == 0)
	    {
		    AddFileSize(FileStat.st_size);
	    }
    }
  }
  else
  {
    if (wxYES == ::wxMessageBox("Item already found.\nWould you like rename this file in the CD layout?", "Name conflict", wxYES_NO | wxICON_QUESTION))
    {
      wxDialog RenameDialog(this, -1, "Rename File", wxDefaultPosition);
      RenameFileInLayoutDialog(&RenameDialog);
      wxTextCtrl* pOldFileNameCtrl = (wxTextCtrl*)FindWindow(ID_TEXTCTRL_OLD_FILENAME);
      wxTextCtrl* pNewFileNameCtrl = (wxTextCtrl*)FindWindow(ID_TEXTCTRL_NEW_FILENAME);
      if (pOldFileNameCtrl && pNewFileNameCtrl)
      {
        pOldFileNameCtrl->Enable(FALSE);
        pOldFileNameCtrl->SetValue(strPath.AfterLast(wxFileName::GetPathSeparator()));
        bool bFileExists = FALSE;
        do
        {
          bFileExists = FALSE;
          RenameDialog.Centre();
          if (wxID_OK == RenameDialog.ShowModal())
          {
            strFilePath = GetCurrentCDLayoutTreePath() + pNewFileNameCtrl->GetValue();
            if (m_CDLayoutMap.find(strFilePath) == m_CDLayoutMap.end())
            {
              // Add the item to the map
              //CDLayoutItem RenamedItem(strFilePath, strPath, CDLayoutItem::LAYOUT_FILE, TRUE);
              //AddToCDLayout(RenamedItem, pListCtrl);
              AddToCDLayout(strFilePath, pListCtrl);
              //m_CDLayoutMap[RenamedItem.GetLayoutPosition()] = RenamedItem;

              //Add the item to the list control
              int nIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), pNewFileNameCtrl->GetValue(), (nFileType == CDLayoutItem::LAYOUT_DIRECTORY) ? 0 : 1);
              pListCtrl->SetItem(nIndex, 2, item.GetOrigin());
            }
            else
            {
              ::wxMessageBox("That file name is already in use.\nPlease select another file name or cancel");
              bFileExists = TRUE;
            }
          }
        } while (bFileExists);
      }
    }
  }
}

void CDLayoutTreeCtrl::AddFileSize(long nFileSize)
{
	m_nTotalSize += nFileSize;
  BurnerFrame* pFrame = (BurnerFrame*)::wxGetTopLevelParent(this);
  if (pFrame)
  {
    wxString strSize = "";
    if (m_nTotalSize > 1048576)  //(1024*1024)
    {
      strSize = wxString::Format("%.2f MB", (float)m_nTotalSize / (float)1048576);//(1024*1024)
    }
    else if (m_nTotalSize > 1024)
    {
      strSize = wxString::Format("%.2f KB", (float)m_nTotalSize / (float)1024);
    }
    else
    {
      strSize = wxString::Format("%ld B", m_nTotalSize);
    }
    pFrame->SetUsedSpaceProgressBar(strSize);
    pFrame->SetUsedSpaceGauge(m_nTotalSize / 1048576);//(1024*1024)
  }
}

void CDLayoutTreeCtrl::PopulateTreeAndListFromPath(const CDLayoutItem& item, wxListCtrl* pListCtrl)
{
  // Fill in the list control with the contents of the directory source similar to how the wxGenericDirCtrl fills its list control
  if (pListCtrl)
  {
    // First find the path to traverse
    // This is done by getting the current tree path and "backing up" to the latest mount point
    // Then append on the difference and use that path as the path to traverse
    // The BurnerCDLayoutTraverser must have a pointer to the CDLayoutTreeCtrl or CDLayoutMap
    //    so that it can know not to add files/directories marked as EXCLUDED
    //wxDir dir;
    //dir.Open(GetPath());
    //BurnerCDLayoutTraverser

    wxDir dir;
    dir.Open(item.GetOrigin());
    // Add entries to the tree control
    CDLayoutTreePopulatingTraverser TreePopTraverse(item, this);
    dir.Traverse(TreePopTraverse);

    // Add entries to the list control
    BurnerCDLayoutTraverser DirTraverse(pListCtrl);
    dir.Traverse(DirTraverse);
  }
}

void CDLayoutTreeCtrl::ChangeCDLayoutDir(wxListCtrl* pListCtrl)
{
  // Clear out the CD layout wxListCtrl
  // Get the CD layout directory from the TreeDnDSource
  // Loop through the keys of the m_CDLayoutMap
  // ONLY CHECK m_CDLayoutMap entries marked as INCLUDE
  // Find the closest match ("/data/CD1/test" is closer to "/data/CD1/test/sub/dir" than "/data/CD1" is)
  // Append the difference between the matching map key and the actual CD layout directory
  // Have the CDLayout wxListCtrl populate itself from the matching map key + appended difference
  // The above will populate the wxListCtrl from the actual source directory structure
  // Now we have to get any additional entries listed in the m_CDLayoutMap
  // and remove any of those entries listed as EXCLUDE
  if (pListCtrl)
  {
    pListCtrl->DeleteAllItems();
    wxString strCurrentLayoutPath = GetCurrentCDLayoutTreePath();
    CDLayoutMap::iterator start = m_CDLayoutMap.begin();
    CDLayoutMap::iterator end = m_CDLayoutMap.end();
    wxString strClosestMatch = "";
    while (start != end)
    {
      if ((*start).second.GetFileType() == CDLayoutItem::LAYOUT_DIRECTORY)
      {
        wxString strLayoutPathMapKey = (*start).first;
        //long nLayoutPathMapKeyLength = (*start).first.Length();
        //wxString strCurrentLayoutPathPrefix = strCurrentLayoutPath.Left((*start).first.Length());
        wxString strLayoutPathMapKeyPrefix = strLayoutPathMapKey.Left(strCurrentLayoutPath.Length());
        //long nClosestMatchLength = strClosestMatch.Length();

        if (((*start).first.CmpNoCase(strCurrentLayoutPath.Left((*start).first.Length())) == 0) &&
            (*start).first.Length() > strClosestMatch.Length())
        //if ((strCurrentLayoutPath.CmpNoCase(strLayoutPathMapKeyPrefix) == 0) &&
        //  strLayoutPathMapKeyPrefix.Length() > strClosestMatch.Length())
        {
            strClosestMatch = (*start).first;
        }

				// Add any directory entries from the m_CDLayoutMap that reside in this directory
				wxString strPathToMatch;
				if ((*start).first == wxFileName::GetPathSeparator())
				{
					strPathToMatch = "";
				}
				else
	 			{
 					strPathToMatch = (*start).first.BeforeLast(wxFileName::GetPathSeparator());
				}
				if (strPathToMatch == strCurrentLayoutPath.BeforeLast(wxFileName::GetPathSeparator()))
				{
  	  	   int nIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), (*start).second.GetLayoutPosition().AfterLast(wxFileName::GetPathSeparator()), 0);
    	  	 pListCtrl->SetItem(nIndex, 2, (*start).second.GetOrigin());
				}
      }
      else if ((*start).second.GetFileType() == CDLayoutItem::LAYOUT_FILE) // This is a file entry
      {
        // Check if this file entry was explicitly added to this folder
        wxString strItemDirectory = (*start).first.BeforeLast(wxFileName::GetPathSeparator());
        strItemDirectory += wxFileName::GetPathSeparator();
        if (strItemDirectory.CmpNoCase(strCurrentLayoutPath) == 0)
        {
          int nIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), (*start).second.GetLayoutPosition().AfterLast(wxFileName::GetPathSeparator()), 1);
          pListCtrl->SetItem(nIndex, 2, (*start).second.GetOrigin());
        }
      }
      start++;
    }
    if (strClosestMatch != "")
    {
      // Now append the difference between the directories to the directory source
      // Have the list control populate itself from this directory source
      wxString strSourcePath = m_CDLayoutMap[strClosestMatch].GetOrigin();
      wxString strAppendedPart = strCurrentLayoutPath.Mid(strClosestMatch.Length());
      wxString strSeparator = wxFileName::GetPathSeparator();
      if (!strAppendedPart.StartsWith(strSeparator))
        strAppendedPart.Prepend(strSeparator);
      wxString strFullPath = strSourcePath + strAppendedPart;
      
      // Fill the list control from this path
      if (wxFileName::DirExists(strFullPath))
      {
        wxDir dir;
        dir.Open(strFullPath);
        BurnerCDLayoutTraverser DirTraverse(pListCtrl);
        dir.Traverse(DirTraverse);
      }
    }

		// Resize the columns
    if (pListCtrl->GetItemCount() > 0)
    {
      pListCtrl->SetColumnWidth(0, -1);
      pListCtrl->SetColumnWidth(2, -1);
    }
  }
}

void CDLayoutTreeCtrl::DisplayLayoutMap()
{
  wxDialog DisplayLayoutMapDialog(this, -1, "Layout Map", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
  ShowCDLayoutDialog(&DisplayLayoutMapDialog);
  wxListCtrl* pListCtrl = (wxListCtrl*)FindWindow(ID_LISTCTRL_CD_LAYOUT);
  if (pListCtrl)
  {
    pListCtrl->InsertColumn(0, "Layout Position");
    pListCtrl->InsertColumn(1, "Origin");
    pListCtrl->InsertColumn(2, "File Type");
    pListCtrl->InsertColumn(3, "Include?");
  
    CDLayoutMap::iterator start = m_CDLayoutMap.begin();
    CDLayoutMap::iterator end = m_CDLayoutMap.end();
  
    while (start != end)
    {
        int nIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), (*start).second.GetLayoutPosition());
        pListCtrl->SetItem(nIndex, 1, (*start).second.GetOrigin());
        pListCtrl->SetItem(nIndex, 2, ((*start).second.GetFileType() == CDLayoutItem::LAYOUT_FILE) ? "File" : "Directory");
        pListCtrl->SetItem(nIndex, 3, "Yes");
        start++;
    }

    // Now add the exclude items
    start = m_CDLayoutExcludeMap.begin();
    end = m_CDLayoutExcludeMap.end();
  
    while (start != end)
    {
        int nIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), (*start).second.GetLayoutPosition());
        pListCtrl->SetItem(nIndex, 1, (*start).second.GetOrigin());
        pListCtrl->SetItem(nIndex, 2, ((*start).second.GetFileType() == CDLayoutItem::LAYOUT_FILE) ? "File" : "Directory");
        pListCtrl->SetItem(nIndex, 3, "No");
        start++;
    }
    if (pListCtrl->GetItemCount() > 0)
    {
      pListCtrl->SetColumnWidth(0, -1);
      pListCtrl->SetColumnWidth(1, -1);
      pListCtrl->SetColumnWidth(2, wxLIST_AUTOSIZE_USEHEADER);
      pListCtrl->SetColumnWidth(3, wxLIST_AUTOSIZE_USEHEADER);
    }
  }
  DisplayLayoutMapDialog.Centre();
  DisplayLayoutMapDialog.ShowModal();
}

void CDLayoutTreeCtrl::CreateTreeDirectory()
{
  wxTreeItemId item = GetSelection();
  wxTreeItemId NewItem = AppendItem(item, "New Directory", 0, 1);
  Expand(item);
  SelectItem(NewItem);
  EditLabel(NewItem);

  // This may not change the m_CDLayoutMap at all.  A side effect to not adding it to the map unless
  //  Files/Directories are added to the new directory is that it won't be added to the CD image
  //  Since it would be empty anyway, I'm okay with this (for now).

}

void CDLayoutTreeCtrl::DeleteTreeDirectory()
{
  //Figure out if any items in the m_CDLayoutMap need to be deleted
  wxString strLayoutPosition = GetCurrentCDLayoutTreePath();
  wxArrayString ItemsToDelete;  // Use an array of map keys that we will loop through AFTER iterating the list here
	wxArrayString ExcludedItemsToReAdd;	// array of map keys of excluded items that have to be "readded" to the file space to be readded to the layout
	wxString strDirectoryItself = "";
  wxString strNewExcludeItem = "";
  if (strLayoutPosition.Right(1) == wxFileName::GetPathSeparator())
	{
		strDirectoryItself = strLayoutPosition;
    strDirectoryItself.RemoveLast(); //Trim off the trailing slash or backslash
	}
  CDLayoutMap::iterator start = m_CDLayoutMap.begin();
  CDLayoutMap::iterator stop = m_CDLayoutMap.end();
  while (start != stop)
  {
    if ((*start).first.StartsWith(strLayoutPosition))
      ItemsToDelete.Add((*start).first);
		else if ((*start).first == strDirectoryItself)
      ItemsToDelete.Add((*start).first);
			
    start++;
  }

  long nSize = 0;

/*
  if (m_CDLayoutMap.find(strLayoutPosition) != m_CDLayoutMap.end())
  {
		long nDiskSpaceUsed = GetDiskSpaceUsed(m_CDLayoutMap[strLayoutPosition].GetOrigin());
		cout << "Removing: " << nDiskSpaceUsed << " : " << m_CDLayoutMap[strLayoutPosition].GetOrigin() << endl;
    nSize += nDiskSpaceUsed;
  }
  else  // Now back up and find the "closest match" directory
*/
  if ((m_CDLayoutMap.find(strLayoutPosition) == m_CDLayoutMap.end()) && (m_CDLayoutMap.find(strDirectoryItself) == m_CDLayoutMap.end()))
  {
    wxString strOrigin = GetOriginThroughParentMapEntry(strLayoutPosition);
//		long nDiskSpaceUsed = GetDiskSpaceUsed(strOrigin);
//		cout << "Removing: " << nDiskSpaceUsed << " : " << strOrigin << endl;
//    nSize += nDiskSpaceUsed;

    // In this case, this is a subdirectory of a specified mount point
    // Do to this fact, we must ADD a map item indicating that this file/directory is to be excluded
		int nFileType = wxFileName::DirExists(strOrigin) ? CDLayoutItem::LAYOUT_DIRECTORY : CDLayoutItem::LAYOUT_FILE;
    CDLayoutItem ExcludeItem(strLayoutPosition, strOrigin, nFileType);
		m_CDLayoutExcludeMap[strLayoutPosition] = ExcludeItem;
    strNewExcludeItem = strLayoutPosition;
  }

  // Now go through the list of items to delete
  // In this loop we should do the following
  //  1) Check if this item was INCLUDED/EXCLUDED
  //  2) If included, then we should get the file size or directory disk usage
  //  3) Call AddFileSize with the amount as a negative number to remove this disk usage
  // Where this get's really tricky is getting the directory disk usage for INCLUDED directories
  //  but subtracting out the disk usage of any files/directories under this directory that
  //  were explicitely EXCLUDED
  //  Otherwise we could end up with a negative space used value
  for (unsigned int i=0; i < ItemsToDelete.GetCount(); i++)
  {
    long nCurrentItemSize = GetDiskSpaceUsed(m_CDLayoutMap[ItemsToDelete[i]].GetOrigin());
    nSize += nCurrentItemSize;

    m_CDLayoutMap.erase(ItemsToDelete[i]);
  }

	// Now remove the disk space total of the excluded items
  start = m_CDLayoutExcludeMap.begin();
  stop = m_CDLayoutExcludeMap.end();
  while (start != stop)
  {
    if ((*start).first.StartsWith(strLayoutPosition))
      ExcludedItemsToReAdd.Add((*start).first);
		else if ((*start).first == strDirectoryItself)
      ExcludedItemsToReAdd.Add((*start).first);
			
    start++;
  }
  for (unsigned int j=0; j < ExcludedItemsToReAdd.GetCount(); j++)
  {
		wxString strLayoutPos = ExcludedItemsToReAdd[j];
		wxString strOrigin = m_CDLayoutExcludeMap[strLayoutPos].GetOrigin();
//    long nCurrentItemSize = GetDiskSpaceUsed(m_CDLayoutExcludeMap[ExcludedItemsToReAdd[j]].GetOrigin());
    long nCurrentItemSize = GetDiskSpaceUsed(strOrigin);
		//cout << "Removing: " << nCurrentItemSize << " : " << m_CDLayoutExcludeMap[ItemsToDelete[j]].GetOrigin() << endl;
    nSize -= nCurrentItemSize;

    if (strNewExcludeItem != strLayoutPos)
      m_CDLayoutExcludeMap.erase(ExcludedItemsToReAdd[j]);
  }

  AddFileSize(-1 * nSize);  // Now remove this whole branch from the tree

  // Now we need to check and see if we need to create an EXCLUDE entry in the m_CDLayoutMap
  //  This will happen if this is a subdirectory of a live directory that is in the map
  //  So I guess that we have to walk up the tree looking for LAYOUT_DIRECTORY mount points and
  //  see if this needs to be excluded.
  //  This may be similar to how we obtain strClosestMatch in the ChangeCDLayoutDir function

  // Finally delete the tree item
  wxTreeItemId item = GetSelection();
  Delete(item);

  // The following functionality may go in the EVT_TREE_DELETE_ITEM handler
  // If this event is in m_CDLayoutMap, then remove the entry
  // If this event is a subdirectory of an item in m_CDLayoutMap, then another entry must
  //  be made to m_CDLayoutMap excluding this directory from the layout
  // We might want to add an item to the popup menu for "Restore Deleted Directories" which
  //  would let the user readd this directory since it still exists on the hard drive
}

void CDLayoutTreeCtrl::RenameTreeDirectory()
{
  wxTreeItemId item = GetSelection();
  EditLabel(item);
  
  // The following functionality may go in the EVT_TREE_END_LABEL_EDIT handler
  // Checking should be done to insure that duplicate folder names are not accepted.  I'm not sure
  //  how to deal with case sensitive checking versus Joliet/Rockridge images
  // If the label was changed then we have to update the m_CDLayoutMap.
  //  This may mean that we need to have the changed directory ignored in it's directory heirarchy
  //  and have a new item in the map added to custom add this to the layout
  //  Also any items in the map that were below this item must be changed to reflect this change.
  //  On the bright side, since we're only adding "customizations" to the m_CDLayoutMap,
  //  This won't be a huge amount of work (hopefully)
}

void CDLayoutTreeCtrl::Clear()
{
  m_CDLayoutMap.clear();
  m_CDLayoutExcludeMap.clear();
  DeleteAllItems();
  AddRoot(wxFileName::GetPathSeparator(), 0, 1);
	m_nTotalSize = 0;
}

long CDLayoutTreeCtrl::GetDiskSpaceUsed(const wxString& strPath)
{
  long nSize = 0;

  if (wxFileName::DirExists(strPath)) // This is a directory
  {
    wxDir dir;
    dir.Open(strPath);
    CDLayoutSpaceTraverser DirTraverse(&nSize);
    dir.Traverse(DirTraverse);
  }
  else if (wxFileName::FileExists(strPath)) // This is a file
  {
	  wxStructStat FileStat;
	  if (::wxStat(strPath, &FileStat) == 0)
	  {
		  // Get the file size
		  nSize += FileStat.st_size;
	  }
  }

  return nSize;
}

wxString CDLayoutTreeCtrl::GetMostDirectParentMapItem(const wxString& strTreePath)
{
  wxString strClosestMatch = "";
  CDLayoutMap::iterator start = m_CDLayoutMap.begin();
  CDLayoutMap::iterator end = m_CDLayoutMap.end();
  while (start != end)
  {
    if ((*start).second.GetFileType() == CDLayoutItem::LAYOUT_DIRECTORY)
    {
      wxString strLayoutPathMapKey = (*start).first;
      //long nLayoutPathMapKeyLength = (*start).first.Length();
      //wxString strTreePathPrefix = strTreePath.Left((*start).first.Length());
      wxString strLayoutPathMapKeyPrefix = strLayoutPathMapKey.Left(strTreePath.Length());
      //long nClosestMatchLength = strClosestMatch.Length();

      if (((*start).first.CmpNoCase(strTreePath.Left((*start).first.Length())) == 0) &&
          (*start).first.Length() > strClosestMatch.Length())
      //if ((strTreePath.CmpNoCase(strLayoutPathMapKeyPrefix) == 0) &&
      //  strLayoutPathMapKeyPrefix.Length() > strClosestMatch.Length())
      {
          strClosestMatch = (*start).first;
      }
    }
    start++;
  }
  return strClosestMatch;
}

wxString CDLayoutTreeCtrl::GetOriginThroughParentMapEntry(const wxString& strTreePath)
{
  // Now append the difference between the directories to the directory source
  // Have the list control populate itself from this directory source
  wxString strClosestMatch = GetMostDirectParentMapItem(strTreePath);
  wxString strSourcePath = m_CDLayoutMap[strClosestMatch].GetOrigin();
  wxString strAppendedPart = strTreePath.Mid(strClosestMatch.Length());
  wxString strSeparator = wxFileName::GetPathSeparator();
  if (!strAppendedPart.StartsWith(strSeparator))
    strAppendedPart.Prepend(strSeparator);
  wxString strFullPath = strSourcePath + strAppendedPart;
  return strFullPath;
}

wxString CDLayoutTreeCtrl::GetOrigin(const wxString& strPath)
{
  if (m_CDLayoutMap.find(strPath) != m_CDLayoutMap.end())
    return m_CDLayoutMap[strPath].GetOrigin();
  else
    return GetOriginThroughParentMapEntry(strPath);
}

bool CDLayoutTreeCtrl::TreeItemIsEditable(const wxTreeItemId& item)
{
  // Currently, only the root node isn't editable
  return (item != GetRootItem());
}

//----------------------------------------------------------------------------
// BurnerCDLayoutTraverser
//----------------------------------------------------------------------------

wxDirTraverseResult BurnerCDLayoutTraverser::OnFile(const wxString& filename)
{
  if (m_pCDLayoutListCtrl)
  {
    wxFileName file(filename);
		wxString strFileName = file.GetFullName();
		if (m_pCDLayoutListCtrl->FindItem(-1, strFileName) == -1)
		{
	    int nIndex = m_pCDLayoutListCtrl->InsertItem(m_pCDLayoutListCtrl->GetItemCount(), strFileName, 1);
/*
	    wxStructStat FileStat;
  	  if (::wxStat(filename, &FileStat) == 0)
	    {
  	    // Get the file size
    	  wxString strSize;
      	if (FileStat.st_size > 1048576(//(1024*1024)
	      {
  	      strSize = wxString::Format("%.2f MB", FileStat.st_size / 1048576);//(1024*1024)
    	  }
	      else if (FileStat.st_size > 1024)
  	    {
    	    strSize = wxString::Format("%.2f KB", FileStat.st_size / 1024);
	      }
  	    else
    	  {
      	  strSize = wxString::Format("%.ld B", FileStat.st_size);
	      }
  	    m_pCDLayoutListCtrl->SetItem(nIndex, 1, strSize);

	      // Get the permissions
  	    wxString strPermissions = wxString::Format("%c%c%c%c%c%c%c%c%c",
    	              (FileStat.st_mode & S_IRUSR) ? 'r' : '-',   // Owner Read
      	            (FileStat.st_mode & S_IRGRP) ? 'r' : '-',   // Group Read
        	          (FileStat.st_mode & S_IROTH) ? 'r' : '-',   // World Read
          	        (FileStat.st_mode & S_IWUSR) ? 'w' : '-',   // Owner Write
            	      (FileStat.st_mode & S_IWGRP) ? 'w' : '-',   // Group Write
              	    (FileStat.st_mode & S_IWOTH) ? 'w' : '-',   // World Write
                	  (FileStat.st_mode & S_IXUSR) ? 'x' : '-',   // Owner Execute
                  	(FileStat.st_mode & S_IXGRP) ? 'x' : '-',   // Group Execute
	                  (FileStat.st_mode & S_IXOTH) ? 'x' : '-');  // World Execute
  	    m_pCDLayoutListCtrl->SetItem(nIndex, 3, strPermissions);

    	  // Get the ownership

	    }
*/
  	  m_pCDLayoutListCtrl->SetItem(nIndex, 2, filename);
		}
  }
  return wxDIR_CONTINUE;
}


wxDirTraverseResult BurnerCDLayoutTraverser::OnDir(const wxString& dirname)
{
  if (m_pCDLayoutListCtrl)
  {
		wxString strDirName = dirname.AfterLast(wxFileName::GetPathSeparator());
		if (m_pCDLayoutListCtrl->FindItem(-1, strDirName) == -1)
	    m_pCDLayoutListCtrl->InsertItem(m_pCDLayoutListCtrl->GetItemCount(), strDirName, 0);
  }
  return wxDIR_IGNORE;
}

//----------------------------------------------------------------------------
// CDLayoutSpaceTraverser
//----------------------------------------------------------------------------
wxDirTraverseResult CDLayoutSpaceTraverser::OnFile(const wxString& filename)
{
	wxStructStat FileStat;
	if (::wxStat(filename, &FileStat) == 0)
	{
		// Get the file size
		*m_pTotalSize += FileStat.st_size;
	}
	
  return wxDIR_CONTINUE;
}

wxDirTraverseResult CDLayoutSpaceTraverser::OnDir(const wxString& dirname)
{
	// Tell it to continue into the files and subdirectories of this directory
  return wxDIR_CONTINUE;
}


//----------------------------------------------------------------------------
// CDLayoutTreePopulatingTraverser
//----------------------------------------------------------------------------
CDLayoutTreePopulatingTraverser::CDLayoutTreePopulatingTraverser(const CDLayoutItem& StartPoint, wxTreeCtrl* pTree)
{
  m_CDLayoutItemStartPoint = StartPoint;
  m_pTreeCtrl = pTree;
  wxTreeItemId StartTreeBranch = m_pTreeCtrl->GetSelection();
  if (!StartTreeBranch.IsOk())
  {
    StartTreeBranch = m_pTreeCtrl->GetRootItem();
  }
  if (StartTreeBranch.IsOk())
  {
    m_TreeMap[StartPoint.GetOrigin()] = StartTreeBranch;
  }
}

wxDirTraverseResult CDLayoutTreePopulatingTraverser::OnFile(const wxString& filename)
{
  return wxDIR_CONTINUE;
}

wxDirTraverseResult CDLayoutTreePopulatingTraverser::OnDir(const wxString& dirname)
{
  if (m_TreeMap.size() == 0)
  {
    return wxDIR_STOP;    // There is no root tree item to attach to.  Stop now
  }

  wxTreeItemId ParentTreeCtrl;
  // Check if this is the root of the CDLayoutItem
  if (m_CDLayoutItemStartPoint.GetOrigin() == dirname)
  {
    ParentTreeCtrl = m_pTreeCtrl->GetSelection();
  }
  else
  {
    // Look this dirname up in the map
    // If this dirname is not in the map (and is not the root of the CDLayoutItem)
    //    then back up a directory and look that up in the map.
    //  If it still isn't found then move on to the next item
    wxString strParentDir = dirname.BeforeLast(wxFileName::GetPathSeparator());
    // Make a reasonable attempt to get the parent tree item
    if (m_TreeMap.find(strParentDir) != m_TreeMap.end())
    {
      ParentTreeCtrl = m_TreeMap[strParentDir];
    }
    else
    {
      strParentDir = strParentDir.BeforeLast(wxFileName::GetPathSeparator());
      if (m_TreeMap.find(strParentDir) != m_TreeMap.end())
      {
        ParentTreeCtrl = m_TreeMap[strParentDir];
      }
    }
  }
  if (ParentTreeCtrl.IsOk())
  {
    // Append this item as a directory in the tree control
    wxTreeItemId NewTreeItem = m_pTreeCtrl->AppendItem(ParentTreeCtrl, dirname.AfterLast(wxFileName::GetPathSeparator()), 0, 1);

    wxASSERT(NewTreeItem.IsOk());

    // Add this entry to the map
    m_TreeMap[dirname] = NewTreeItem;
  }

  // Return continue to move on into this directory for any more directories
  return wxDIR_CONTINUE;
}

