#ifndef __CD_LAYOUY_TREE_CTRL__
#define __CD_LAYOUY_TREE_CTRL__

#ifdef __GNUG__
    #pragma interface "wxBurn.h"
#endif

// Include wxWindows' headers

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/hashmap.h>
#include <wx/treectrl.h>
#include <wx/listctrl.h>
#include <wx/dir.h>

#include <map>

//----------------------------------------------------------------------------
// CDLayoutItem
//----------------------------------------------------------------------------
class CDLayoutItem : public wxObject
{
public:
    CDLayoutItem(const wxString& strLayoutPosition = "", const wxString& strOrigin = "",
                            int nFileType = LAYOUT_FILE)
    {
        m_strLayoutPosition = strLayoutPosition;
        m_strOrigin = strOrigin;
        m_nFileType = nFileType;
    }

    enum LayoutItemType
    {
        LAYOUT_DIRECTORY = 0,
        LAYOUT_FILE
    };

    CDLayoutItem& operator=(const CDLayoutItem& item);
    void SetLayoutPosition(const wxString& strLayoutPosition) { m_strLayoutPosition = strLayoutPosition; }
    wxString GetLayoutPosition() const { return m_strLayoutPosition; }
    void SetOrigin(const wxString& strOrigin) { m_strOrigin = strOrigin; }
    wxString GetOrigin() const { return m_strOrigin; }
    void SetFileType(int nFileType) { m_nFileType = nFileType; }
    int GetFileType() const { return m_nFileType; }
protected:
    wxString    m_strLayoutPosition;
    wxString    m_strOrigin;
    int             m_nFileType;
};


//WX_DECLARE_STRING_HASH_MAP(CDLayoutItem, CDLayoutMap);
typedef std::map<wxString, CDLayoutItem> CDLayoutMap;

//----------------------------------------------------------------------------
// CDLayoutTreeCtrl
//----------------------------------------------------------------------------
class CDLayoutTreeCtrl : public wxTreeCtrl
{
public:
  CDLayoutTreeCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize, long style = wxTR_HAS_BUTTONS,
      const wxValidator& validator = wxDefaultValidator, const wxString& name = "listCtrl");

  //void AddToCDLayout(CDLayoutItem& item, wxListCtrl* pListCtrl);
  void AddToCDLayout(wxString& strPath, wxListCtrl* pListCtrl, const wxString& strTreePath  = "");
  wxString GetCurrentCDLayoutTreePath();
  wxString GetCDLayoutTreePathFromCoord(wxCoord x, wxCoord y);
	void SelectItemFromHitTest(const wxPoint& pt);
  void PopulateTreeAndListFromPath(const CDLayoutItem& item, wxListCtrl* pListCtrl);
  void ChangeCDLayoutDir(wxListCtrl* pListCtrl);
  CDLayoutMap* GetLayoutMap() { return &m_CDLayoutMap; }
  CDLayoutMap* GetLayoutExcludeMap() { return &m_CDLayoutExcludeMap; }
  void DisplayLayoutMap();
	void AddFileSize(long nFileSize);
	long GetTotalSize() { return m_nTotalSize; }
  void SetTotalSize(long nSize) { m_nTotalSize = nSize; }
	void CreateTreeDirectory();
	void DeleteTreeDirectory();
	void RenameTreeDirectory();
  void Clear();
	long GetDiskSpaceUsed(const wxString& strPath);
  wxString GetMostDirectParentMapItem(const wxString& strTreePath);
  wxString GetOriginThroughParentMapEntry(const wxString& strTreePath);
  wxString GetOrigin(const wxString& strPath);
  bool TreeItemIsEditable(const wxTreeItemId& item);

protected:
  CDLayoutMap       m_CDLayoutMap;
  CDLayoutMap       m_CDLayoutExcludeMap;
  long              m_nTotalSize;
};

//----------------------------------------------------------------------------
// BurnerCDLayoutTraverser
//----------------------------------------------------------------------------
class BurnerCDLayoutTraverser : public wxDirTraverser
{
public:
  BurnerCDLayoutTraverser(wxListCtrl* pListCtrl) { m_pCDLayoutListCtrl = pListCtrl; }

  virtual wxDirTraverseResult OnFile(const wxString& filename);
  virtual wxDirTraverseResult OnDir(const wxString& dirname);

protected:
  wxListCtrl* m_pCDLayoutListCtrl;
};

//----------------------------------------------------------------------------
// CDLayoutSpaceTraverser
//----------------------------------------------------------------------------
class CDLayoutSpaceTraverser : public wxDirTraverser
{
public:
  CDLayoutSpaceTraverser(long* pTotalSize) { m_pTotalSize = pTotalSize; }

  virtual wxDirTraverseResult OnFile(const wxString& filename);
  virtual wxDirTraverseResult OnDir(const wxString& dirname);

protected:
  long* m_pTotalSize;
};

//WX_DECLARE_STRING_HASH_MAP(wxTreeItemId, SearchTreeMap);
typedef std::map<wxString, wxTreeItemId> SearchTreeMap;

//----------------------------------------------------------------------------
// CDLayoutTreePopulatingTraverser
//----------------------------------------------------------------------------
class CDLayoutTreePopulatingTraverser : public wxDirTraverser
{
public:
  CDLayoutTreePopulatingTraverser(const CDLayoutItem& StartPoint, wxTreeCtrl* m_pTreeCtrl);

  virtual wxDirTraverseResult OnFile(const wxString& filename);
  virtual wxDirTraverseResult OnDir(const wxString& dirname);

protected:
  CDLayoutItem m_CDLayoutItemStartPoint;
  wxTreeCtrl* m_pTreeCtrl;
  SearchTreeMap m_TreeMap;

};


#endif // __CD_LAYOUY_TREE_CTRL__
