#include "wxBurnDnD.h"

#include <wx/filename.h>

// ----------------------------------------------------------------------------
// Notifications called by the base class
// ----------------------------------------------------------------------------
bool TreeDnDFile::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
{
  size_t nFiles = filenames.GetCount();
  wxTreeItemId SelectedTreeItem = m_pOwner->GetSelection();
  for ( size_t n = 0; n < nFiles; n++ )
  {
    int nItemType = -1;
    if (wxFileName::DirExists(filenames[n]))  // Directory
    {
      nItemType = CDLayoutItem::LAYOUT_DIRECTORY;
    }
    else if (wxFileName::FileExists(filenames[n]))  // File
    {
      nItemType = CDLayoutItem::LAYOUT_FILE;
    }
    else  // The current file is neither an existing file or directory so move on
      continue;

    //CDLayoutItem item(m_pOwner->GetCDLayoutTreePathFromCoord(x, y), filenames[n], nItemType, TRUE);
    //m_pOwner->AddToCDLayout(item);
    
    // The call the AddToCDLayout calls SelectItem of the tree item that it creates.  This causes subsequent
    //  iterations through the loop to be children of previous iterations.  To fix this we "re-select" the
    //  originally selected item
    //if (SelectedTreeItem.IsOk())
      m_pOwner->SelectItem(SelectedTreeItem);
    m_pOwner->AddToCDLayout(filenames[n], m_pListCtrl, m_pOwner->GetCDLayoutTreePathFromCoord(x, y));
  }

  return TRUE;
}

bool ListDnDFile::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
{
  size_t nFiles = filenames.GetCount();
  for ( size_t n = 0; n < nFiles; n++ )
  {
    int nItemType = -1;
    if (wxFileName::DirExists(filenames[n]))  // Directory
    {
      nItemType = CDLayoutItem::LAYOUT_DIRECTORY;
    }
    else if (wxFileName::FileExists(filenames[n]))  // File
    {
      nItemType = CDLayoutItem::LAYOUT_FILE;
    }
    else  // The current file is neither an existing file or directory so move on
      continue;

    //CDLayoutItem item(m_pNavTree->GetCurrentCDLayoutTreePath(), filenames[n], nItemType, TRUE);
    //m_pNavTree->AddToCDLayout(item);
    m_pNavTree->AddToCDLayout(filenames[n], m_pOwner);
  }

  return TRUE;
}

//----------------------------------------------------------------------------
// BurnerAvailFilesTraverser
//----------------------------------------------------------------------------

wxDirTraverseResult BurnerAvailFilesTraverser::OnFile(const wxString& filename)
{
  if (m_pAvailFilesListCtrl)
  {
    wxFileName file(filename);
    int nIndex = m_pAvailFilesListCtrl->InsertItem(m_pAvailFilesListCtrl->GetItemCount(), file.GetFullName(), 1);
/*
    wxStructStat FileStat;
    if (::wxStat(filename, &FileStat) == 0)
    {
      // Get the file size
      wxString strSize;
      if (FileStat.st_size > 1048576(//(1024*1024)
      {
        strSize = wxString::Format("%.2f MB", FileStat.st_size / 1048576);//(1024*1024)
      }
      else if (FileStat.st_size > 1024)
      {
        strSize = wxString::Format("%.2f KB", FileStat.st_size / 1024);
      }
      else
      {
        strSize = wxString::Format("%.ld B", FileStat.st_size);
      }
      m_pAvailFilesListCtrl->SetItem(nIndex, 1, strSize);

      // Get the permissions
      wxString strPermissions = wxString::Format("%c%c%c%c%c%c%c%c%c",
                  (FileStat.st_mode & S_IRUSR) ? 'r' : '-',   // Owner Read
                  (FileStat.st_mode & S_IRGRP) ? 'r' : '-',   // Group Read
                  (FileStat.st_mode & S_IROTH) ? 'r' : '-',   // World Read
                  (FileStat.st_mode & S_IWUSR) ? 'w' : '-',   // Owner Write
                  (FileStat.st_mode & S_IWGRP) ? 'w' : '-',   // Group Write
                  (FileStat.st_mode & S_IWOTH) ? 'w' : '-',   // World Write
                  (FileStat.st_mode & S_IXUSR) ? 'x' : '-',   // Owner Execute
                  (FileStat.st_mode & S_IXGRP) ? 'x' : '-',   // Group Execute
                  (FileStat.st_mode & S_IXOTH) ? 'x' : '-');  // World Execute
      m_pAvailFilesListCtrl->SetItem(nIndex, 3, strPermissions);

      // Get the ownership

    }
*/
    m_pAvailFilesListCtrl->SetItem(nIndex, 2, file.GetModificationTime().Format());
  }
  return wxDIR_CONTINUE;
}


wxDirTraverseResult BurnerAvailFilesTraverser::OnDir(const wxString& dirname)
{
  if (m_pAvailFilesListCtrl)
  {
    m_pAvailFilesListCtrl->InsertItem(m_pAvailFilesListCtrl->GetItemCount(), dirname.AfterLast(wxFileName::GetPathSeparator()), 0);
  }
  return wxDIR_IGNORE;
}

